procedure lrsart(txtfile,outimg)
# msetscr$lrsart.cl
# This task takes a text file of coordinates and generates an artificial LRS
# image at a position angle of 0.0 degrees

# Revision History:
# v.0.0.3 2002-Jun-13 (JRT)
#  - First version based on genfakeglob-0.0.2.cl


## Parameters
char  txtfile  {"",prompt="Input file containing dRA, dDEC, mag"}
char  outimg   {"",prompt="Output image name"}
real  fcenra   {0.,prompt="Central RA in hours"}
real  fcendec  {0.,prompt="Central DEC in degrees"}
real  fepoch   {2000.0,prompt="Epoch of central coordinates"}
real  fwhm     {2.,prompt="Object FWHM in arcsec"}
real  magcen   {24.,prompt="Mean output magnitude"}
bool  clobber  {no,prompt="Clobber an existing output image"}

begin

#Read in RA (HH:MM:SS), DEC(HH:MM:SS)
# How about Central RA, DEC and offset

#read central RA in hrs
#read central dec in deg
#read plate scale in as/pix
#read all offsets as pairs dra, ddec

#generate CD values
#assume scale size is 1, and xra=568, yra=512


## Variables
# Input parameter copies
char    n_txtfile,n_outimg
real    n_fcenra,n_fcendec,n_fcendecr,n_fepoch
real    n_fwhm,n_magcen
bool    n_clobber
char    verstring
#
real    D2R 
real    n_mag
real	o_ra, o_dec
real	dra, ddec
real	platesc,invplsc,fwhm
real    o_pixx, o_pixy
char    temp
real    fmszx, fmszy, offsetx, offsety
real    r_hd, r_m, r_s
char    s_coord
#real    offsetRA, offsetDEC

# Set the version string
verstring     = "v.0.0.3 2002-Jun-13"

# Load required packages
noao.artdata

#Starup message
printf("lrsart.cl -- %s\n",verstring)
printf("This generates a noiseless image so display with:\n")
printf("display outputfilename 1 zsc-\n")
printf("-----------------------------------------------------------------\n\n")

## Copy parameter values
n_txtfile  = txtfile
n_outimg   = outimg
n_fcenra   = fcenra * 15.
n_fcendec  = fcendec
n_fepoch   = fepoch
n_fwhm     = fwhm
n_magcen   = magcen
n_clobber  = clobber
# msetup parameters
D2R        = pi / 180.
n_fcendecr = n_fcendec * D2R     # Convert field center dec to radians
#platesc    = 1.                 # in pix/as
#platesc    = 2.1524             # in pix/as
platesc    = pixpas / 2.         #unbinned pixels/as divided by 2x2 binning
invplsc    = 1. / platesc
#n_fwhm     = 2.0                # FWHM of stellar images in arcsec
#GCcenRA   = 259.280294
#GCcenDEC  = 43.136524
#offsetRA  = 284.5 - (3600*(259.2666667 - GCcenRA)*platesc)
#offsetDEC =  256.5 - (3600*(43.0336111 - GCcenDEC)*platesc)
#n_outimg = "M92f-offset"
fmszx      = 1136.
fmszy      = 1024.
offsetx    = (fmszx / 2.) + 0.5
offsety    = (fmszy / 2.) + 0.5


## Test for existence of input and output files
if (n_clobber && access(n_outimg)) {
  imdelete(n_outimg, verify+)
}
if (access(n_outimg)) {
  print("Output image "//n_outimg//" exists.  Task stopped.")
  bye
}
if (!access(n_txtfile)) {
  print("Input data file "//n_txtfile//" doesn't exist. Task stopped.")
  bye
}

## Make temporary file location
temp=mktemp("tmp$mmart-")


## Output pixel positions to tempoarary file
mlist3 = n_txtfile
#while (fscan(mlist3,RA,DEC,dRA,dDEC,rapm,rapme,decpm,decpme,prob,npos,mag) != EOF) {
while (fscan(mlist3,o_ra,o_dec,n_mag) != EOF) {
  #pixx   = -1.0*(dRA-offsetRA)*cos(DEC)*platesc
  #pixy   = (dDEC-offsetDEC)*platesc
  #printf ("%6.2f  %6.2f  %4.1f\n",-1.0*dRA*platesc,dDEC*platesc,n_mag)
  #printf ("%6.2f  %6.2f  %4.1f\n",pixx,pixy,n_mag,>> temp)
  #printf ("%6.2f  %6.2f  %4.1f\n",-1.0*dRA*platesc,dDEC*platesc,n_mag,>>temp)
  ddec       = o_dec - n_fcendec
  o_pixy     = 3600. * ddec * platesc
  o_ra      *= 15.
  dra        = (o_ra  - n_fcenra) * cos(n_fcendecr)
  o_pixx     = -3600.0 * dra * platesc
  printf ("%7.2f  %7.2f  %5.2f\n",o_pixx,o_pixy,n_mag,>>temp)
}


## Generate output file
# Note this is a binned 1136x1024 image which modmos will trim down
mkobjects (n_outimg,title="Artifical Setup Frame",
           #ncols=1136,nlines=1024,header="msetscr$lrsart-header.dat",
           ncols=int(fmszx),nlines=int(fmszy),
           xoffset=offsetx,yoffset=offsety,
           objects=temp,
           star="gaussian",radius=(n_fwhm*platesc/2),
           distance=1.,exptime=1.,magzero=n_magcen,rdnoise=0.)
# Add header parameters to output file
#hedit (n_outimg,"POSANGLE",0.,add+,ver-,show-)
hedit (n_outimg,"CD1_1",(-1.0*invplsc/3600.0),add+,ver-,show-)
hedit (n_outimg,"CD2_2",(invplsc/3600.0),add+,ver-,show-)
hedit (n_outimg,"CD1_2",0.,add+,ver-,show-)
hedit (n_outimg,"CD2_1",0.,add+,ver-,show-)
hedit (n_outimg,"CTYPE1","RA---TAN",add+,ver-,show-)
hedit (n_outimg,"CTYPE2","DEC--TAN",add+,ver-,show-)
hedit (n_outimg,"CRPIX1",offsetx,add+,ver-,show-)
hedit (n_outimg,"CRPIX2",offsety,add+,ver-,show-)
hedit (n_outimg,"CRVAL1",n_fcenra,add+,ver-,show-)
hedit (n_outimg,"CRVAL2",n_fcendec,add+,ver-,show-)
hedit (n_outimg,"EQUINOX",n_fepoch,add+,ver-,show-)
hedit (n_outimg,"RADECSYS","FK5",add+,ver-,show-)
# Coordinate stuff
hedit (n_outimg,"EPOCH",n_fepoch,add+,ver-,show-)
n_fcenra /= 15.
printf ("%12.2h\n",n_fcenra)  | scan(s_coord)
hedit (n_outimg,"RA",s_coord,add+,ver-,show-)
printf ("%12.2h\n",n_fcendec) | scan(s_coord)
hedit (n_outimg,"DEC",s_coord,add+,ver-,show-)
hedit (n_outimg,"MMARTVER","lrsart.cl -- "//verstring,add+,ver-,show-)

## Clean up the mess
#delete (temp)

end
