procedure makemos(name)
# msetscr$makemos.cl
# This task can take an ascii file or an image plus curser input, and will
# determine the proper corrections to apply to the LRS MOS unit for proper
# positioning of the slitlets.  Output is written to the appropriate
# configuration file.

# Revision History:
# v.5.0.0 2002-Jul-16 (JRT)
#  -Release version
# v.4.3.2 2002-Jun-10 (JRT)
#  -Added marking of reference objects
#  -Fixed marking colors
# v.4.3.1 2002-Jun-5 by J. Tufts
#  -Major code rewrite to remove some cruft and redundancy, it should be more
#   readable now.
#  -added mkm output file which can be used for input in other msetup progs.
#  -mkm-*.fits is a standard .fits extension file which has three tables at
#   the end. A slitlet position table, a reference object position table, and
#   a distortion and mos information table. Try tprint mkm-filnm.fits[1] or
#   [2] or [3], and display mkm-filnm.fits[0]
#  -output file name is automatically generated from input if it is not spec'd.
#  -Moved finder chart generation to a new program: mfinder
#  -Reference object selection is now in makemos and up to 5 objects can be
#   chosen, furthermore radial plots can be done during this process
#  -cfgfile and mkmfile are no longer output.
#  -added a version string variable which is used consistently throughout
#  -temporarily removed ability to generate from a text file, eventually this
#   will be replaced with option of text or mkm file.
#  -slitlet mask transparency was strongly increased from 0.17 to 0.90
#  BUG: Image over display is not working properly, it doesn't delete old slits
# v.4.3.0 2002-Jan-30 by J. Tufts (based on E. Hooper's work)
#  -modified parameters so a blank mkmfile and cfgfile result in a name
#   generated from the input filename
#  -changed version numbering scheme to standard practice
#    - changes in major version reflect major design/usage structure changes
#    - changes in minor v. are generation changes made after new LRS setup
#    - even minor numbers are release versions, odd minors are development
#    - changes in subminor v. are solely bugfixes
#  -corrected start up message
#  -modified program to work with msetup where critical mos params are set
#  -task now defined in msetup
# v.4.1.0 2002-Feb-11 by J. Tufts
#  -distribution version with only working functionality.
#  -fixed startup message
# v.04.01 2001-Dec-18 by Marcel Bergmann
#  -Modified to accept non-HET/LRS setup images (assumes perfect astrometry) 
#   Applies distortion correction to ref,slit positions of finder chart
#  -Also took the distortion coefficients out of the task parameters (now vars)
# v.03.01 2001-Nov-20 by J. Tufts
#  -works better with ds9, and correctly shifts images when making a finder
# v.03.00 2001-Nov-7 by G. Hill
#  -following realignment of LRS and MOS in Oct 2001
#  -Changed to get PA from header and put in config file and on finder 
# v.02.00 2001-Jan-30 by Keith Thompson 
#  -add ability to make finder chart after end  (if inputtype=image)
# v.01.01 2000-Nov-23 by G. Hill
#  -Modified coefficients and home positions on for recommissioned LRS 
#   with v2 CCD controller (amode).
#  -modified to use amode format
# v.01.00 2000-May-08 by Marcel Bergmann UT Austin (marcel@astro.as.utexas.edu)
#  -first release version

# Release Notes:
#  -Current version only works for imt38
#   only 2x2 binned full format images are properly supported in image mode
#   modified Nov 00 to work with amode data ONLY

# Usage Note:
#  BUG: name must be the full file name including extension

# To do:
#  -check that sign of distortion correction has been applied properly 
#  -imshifting an HET image does not update RA/DEC of center.
#   also, the distortion corrections applied will NOT be correct b/c of shift.
#  -fix distortion coefficient for binned/unbinned modes
#  -put mos slitlet positions in msetup file in unbinned mode
#  -allow use of DS9's box drawing capability.

# makemos parameters
char name     {"",prompt="Input text or image file (w/ extension)"}
char intype   {"image",enum="text|image",
	       prompt="Type of input file (text|image)"}
int  binning  {2,prompt="Binning used for x positions in text file"}
char mkmfile  {"",prompt="Name of output makemos data file"} 
char cfgfile  {"",prompt="Name of output configuration file"}
char projid   {"HETxx-y-zzz",prompt="Project ID"}
char objname  {"auto",prompt="Object/Field Name"}
real xshift   {0,prompt="Number of pixels to shift in x"}
real yshift   {0,prompt="Number of pixels to shift in y"}
real disp_z1  {0,prompt="z1 for display (z1=0,z2=0 for auto)"}
real disp_z2  {0,prompt="z2 for display (z1=0,z2=0 for auto)"}
bool clobber  {no, prompt="Clobber old output files"}
char outname  {"",prompt="Output parameter (Do not edit)"}
#We now use mlist[1,2] defined in msetup
#struct* list {"",prompt="list directed struct. DO NOT CHANGE"}

begin

#Variable declarations
char	verstring
char	n_file, n_image, n_imagename
char	n_mkmfile, n_cfgfile
char    n_projid, n_objname
int	n_binning
int	i, sltop, slbot
real	n_scan
int	n_slit
int	n_slit_bottom[14] # bottom of slit pixel position (center of interslit)
real	n_xim	          # number of rows in image
real	n_yim	          # number of columns in image
## Temporary file names
#char	tmppointim        # n_image with interslit regions blacked out
char	tmpdispim         # n_image with non-accessible regions darkened
char	tmplis            # temporary list file for coordinates
char	tmpcoord	  # temporary file with tvmark coordinates
char	n_shiftim         # name for shifted image
char	tmpsltab          # temporary file for slitlet position table 
char	tmpcoeftab        # temp file for distortion coefficient table
char    tmpreftab         # temp file for reference object table
char	tmptdata          # temporary file for table data
##
char	imcalstr          # imcalc string used to calculate tmpdispim
char	n_key             # keypress while assigning slit positions
#bool	continue          # while loop flag for reading cursor slit positions
real	n_xcur            # x position of cursor (pixels)
real	n_ycur            # y position of cursor (pixels)
real	n_xcen            # x position of object centroid
real	n_ycen            # y position of object centroid
#real	n_xslit[13]       # x postion of slitlet (1x1 binning) (text file)
#real	n_xslit_1[13]     # x postion of slitlet (1x1 binning) (image)
#real	n_xslit_2[13]     # x postion of slitlet (2x2 binning) (image)
#real	n_xhome_1[13]     # x positions of slit home positions (1x1 binning)
real	a_xcur[13],a_ycur[13] # X,Y cursor positions
real    a_xcen[13]        # X centroid pos. X=-1(cent), -2(baf), -3(unused)
real    a_ycen[13]        # Y centroid pos. X=-1(cent), -2(baf), -3(unused)
real    a_xdcen[13]       # Distorted X centroid position of slitlet
char    a_key[13]         # key pressed
real	distcor[13]       # Optical distortion correction in pixels, this is
                          # the number _added_ to an x pixel coordinate to
                          # make it an undistorted x coordinate
real    a_slposmm[13]       # Physical position of slitlet in mm
bool    eflag             # error flag     
## Reference object parameters
int	n_rnum            # Number of reference object
real    a_xref[5],a_yref[5] # x,y reference object positions (unused if -1)
real    n_rdisto, n_mdist, n_xdisto, n_ydisto
## Display parameters
real	n_xfrac	          # x fraction for overdisplay
real	n_yfrac	          # y fraction for overdisplay
real	n_dispz1	  # lower display range
real	n_dispz2	  # upper display range
char	dispz1	          # scanned z1
char	dispz2	          # scanned z2
bool	auto_z            # let display find z1, z2 (true if z1=z2=0)
real	n_xshift          # number of pixels to shift in x
real	n_yshift          # number of pixels to shift in y
##
char	n_junk,n_junk2	  # placeholder
#int	iteration         # loop counter
bool	sourcefile	  #
bool	n_clobber
bool	lrsimage          #
bool	a_query
char	n_filetype        # 
#char	n_outunit         # output unit type:  mm or 1x1 pixels
#real	p_pixmm           # number of pixels per mm
## Header parameters
char	n_tele	          # name of telescope used for image acquisition
char	n_inst            # name of instrument used for image acquisition
int	n_xbin	          # x binning of image
int	n_ybin	          # y binning of image
int	refpos_option 
char	target_name, pa_str, ra, dec, epoch
real	pa, pa_mod
real    het_optx, het_opty, het_distcoeff

# Set the version string
verstring     = "v.5.0.0 2002-Jul-16"

#Starup message
printf("makemos.cl -- %s (valid from 2001-Oct)\n\n",verstring)
printf("Current version only works for imt38. If you have weird behavior:\n")
printf("set stdimage=imt38\n\n")
printf("-----------------------------------------------------------------\n\n")

#p_pixmm=20.493 #Eric's new number is 20.518 from pinhole grid measurements
#p_pixmm is now read from the msetup script
#p_pixmm     = pixpmm

n_image     = name
n_filetype  = intype
n_binning   = binning
n_mkmfile   = mkmfile
n_cfgfile   = cfgfile
n_projid    = projid
n_objname   = objname
n_xshift    = xshift
n_yshift    = yshift
n_dispz1    = disp_z1
n_dispz2    = disp_z2
n_clobber   = clobber
het_optx    = ocent[1]/n_binning # assumed center of LRS optical distortion
het_opty    = ocent[2]/n_binning # assumed center of LRS optical distortion


n_imagename = n_image
n_file      = n_image
# Automatically generate and output filename
if (n_mkmfile == "") n_mkmfile = "mkm-"//n_file
if (n_cfgfile == "") n_cfgfile = (substr(n_file,1,strlen(n_file)-4))//".cfg"
outname = n_mkmfile

## Determine if we are to use automatic z scaling
if( n_dispz1 == 0 && n_dispz2 == 0 )  auto_z = yes
else                                  auto_z = no

## Check to see that the input file exists and has the full extension included
if (!access(n_image)) {
  printf("ERROR: %s does not exist.\n Try using a full name with extension\n",
         n_image)
  bye
}

if (n_clobber && access(n_mkmfile)) {
  printf("[If %s not deleted, program will append]\n", n_mkmfile)
  delete(n_mkmfile, verify+)
}
if (!n_clobber && access(n_mkmfile)) {
  print("Configuration file "//n_mkmfile//" exists.  Task stopped.")
  bye
}

if(n_filetype == "image")  sourcefile = no
if(n_filetype == "text")   sourcefile = yes


## initialize the x_slit positions and distcors
# BUG: What does this mean?
for(i=1;i<=13;i+=1) {
  a_xcen[i]    = -2.  # Default to baffled position
  a_ycen[i]    = -2.
  a_xdcen[i]   = -2.
  a_key[i]     = "*"
  a_xcur[i]    = 0.
  a_ycur[i]    = 0.
  a_slposmm[i] = ((n_binning*slbafp)-sze[i])*mmppix
  distcor[i]   = 0
}

# #*********** Modify below so it is not repeated at the end of the script ***
# # if input is a text file, read it in and produce the appropriate config file
# if(sourcefile) {
# mlist3=n_file
#  if(fscan(mlist3,n_slit,n_scan)!=2) {
#    #if (n_slit == INDEF || n_scan == INDEF) {
#   print("Text file should contain two columns:  slit number and star position (pix)")
#   bye
#  }
#  else {}
#  mlist3=n_file
#  while(fscan(mlist3,n_slit,n_scan)!=EOF) {
#  if(n_binning==1) {
#   n_xslit[n_slit] = n_scan
#  } else {		# nbinning=2
#   n_xslit[n_slit] = 2 * n_scan
#  }
# # distcor[n_slit] = (coeff_x3[n_slit] * (n_xslit[n_slit]**3)) + coeff_x2[n_slit] * (n_xslit[n_slit]**2) + coeff_x1[n_slit] * (n_xslit[n_slit]) + coeff_x0[n_slit]
#  distcor[n_slit] = (xco3[n_slit] * (n_xslit[n_slit]**3)) + xco2[n_slit] * (n_xslit[n_slit]**2) + xco1[n_slit] * (n_xslit[n_slit]) + xco0[n_slit]
#  if((n_xslit[n_slit] + distcor[n_slit]) < 1300 || (n_xslit[n_slit] + distcor[n_slit]) > 1960) {
#   printf("\nWARNING: Slit %s has final x value out of range (%7.3f); position set to -\n",str(n_slit),(n_xslit[n_slit] + distcor[n_slit])
#  # n_xslit[n_slit]=9999
#   distcor[n_slit]=1000
#   }
#  }
#  printf("#", >> n_cfgfile)
#  date(, >> n_cfgfile)
#  print("#Configuration file created by makemos.cl, written by Marcel Bergmann", >> n_cfgfile)
#  print("#Please send comments/questions to marcel@astro.as.utexas.edu", >> n_cfgfile)
#  print("#All X output corrected for MOS dispersion offset errors", >> n_cfgfile)
#  print("\nInput file: "//n_file)
#  print("#Input file: "//n_file, >> n_cfgfile)
#  print("Input binning: "//n_binning//"x"//n_binning)
#  print("#Input binning: "//n_binning//"x"//n_binning, >> n_cfgfile)
#  print("Output binning: 1x1")
#  print("#Output binning: 1x1", >> n_cfgfile)
#  if(n_outunit=="mm") {
#   print("Output config file written in mm ("//pixpmm//" pix/mm assumed)")
#   print("#Conversion of "//pixpmm//" pix/mm assumed", >> n_cfgfile)
#   print("#\n#Slit\tX_slit (mm)", >> n_cfgfile)
#  } else {
#   print("Output config file written in 1x1 pixels")
#   print("#\n#Slit\t X_slit (pix)", >> n_cfgfile)
#  }
#  print("\nSlit  x_original       x_error  x_final  [home-x_final pix]  [home-x_final mm]")
#  for(i=1;i<=13;i+=1) {
#   if(n_xslit[i]==9999) {
#    printf("%2.0f\t-\t\t-\t-\t\t-\t\t-\n",i)
#    printf("%2.0f\t    -\n",i, >> n_cfgfile)
#   } else if(n_xslit[i]!=9999 && distcor[i]==1000) {
#    printf("%2.0f\t%7.3f\t\t-\t-\t\t-\t\t-\n",i,n_xslit[i])
#    printf("%2.0f\t    -\n",i, >> n_cfgfile)
#   } else {
# #   printf("%2.0f\t%7.3f\t\t%5.3f\t%7.3f\t\t%6.3f\t\t%6.4f\n",i,n_xslit[i],distcor[i],n_xslit[i] + distcor[i], n_xhome_1[i] - n_xslit[i] - distcor[i],(n_xhome_1[i] - n_xslit[i] - distcor[i]) / pixpmm) 
#    printf("%2.0f\t%7.3f\t\t%5.3f\t%7.3f\t\t%6.3f\t\t%6.4f\n",i,n_xslit[i],distcor[i],n_xslit[i] + distcor[i], shm[i] - n_xslit[i] - distcor[i],(shm[i] - n_xslit[i] - distcor[i]) / pixpmm) 
#    if(n_outunit=="pix") {
# #    printf("%2.0f\t%7.3f\n",i, n_xhome_1[i] - n_xslit[i] - distcor[i], >> n_cfgfile)
#     printf("%2.0f\t%7.3f\n",i, shm[i] - n_xslit[i] - distcor[i], >> n_cfgfile)
#    } else {
# #    printf("%2.0f\t%7.3f\n",i, (n_xhome_1[i] - n_xslit[i] - distcor[i]) / pixpmm, >> n_output)
#     printf("%2.0f\t%7.3f\n",i, (shm[i] - n_xslit[i] - distcor[i]) / pixpmm, >> n_cfgfile)
#    }
#   }
#  }
# }  # end if(sourcefile)


# if input is an image, read positions from image cursor, and make config file
# Read all necessary header information into variables
# CAUTION: For HET images, RA and DEC are not fully calibrated...
if(!sourcefile)  {
  imgets (n_imagename,"POSANGLE"); pa = real(imgets.value)
  if (n_objname == "auto") {
    imgets (n_imagename,"OBJECT");   n_objname = imgets.value
  }
  imgets (n_imagename,"EQUINOX");  epoch = imgets.value
  imgets (n_imagename,"CCDSUM");   print(imgets.value) | scan(n_xbin,n_ybin)
  imgets (n_imagename,"TELESCOP"); n_tele = imgets.value
  imgets (n_imagename,"INSTRUME"); n_inst = imgets.value
  if(n_tele == "het" && n_inst == "lrs") {
    lrsimage=yes
    imgets (n_imagename,"RA");     ra = imgets.value
    imgets (n_imagename,"DEC");    dec = imgets.value
  } else {
    lrsimage=no
    printf ("Image was not taken with the HET/LRS.\n")
    printf ("No astrometric distortion corrections will be applied\n")
    # Translate xy pixel coords of image center to RA and DEC
    ## ****** Possibly this should use wcstran ******
    xy2rd(n_imagename,284,256,hms+,>>& "/dev/null")
    ra  = xy2rd.ra
    dec = xy2rd.dec
  }
}

if(!sourcefile)  {

## mos mask:  inter-slit regions
# controls red slit position indictors on image, not the display of the image
n_slit_bottom[1]  = (1 + int(sbt[1])) / 2         #  0
for (i=2;i<14;i+=1) {
  n_slit_bottom[i] = (int(stp[i-1]) + int(sbt[i])) / 2
}
n_slit_bottom[14] = (int(stp[13]) + 512) / 2      # 512+4

## Make temporary storage locations
tmpdispim  = mktemp("tmp$mkm-dispim")
n_shiftim  = mktemp("tmp$mkm-shift")
tmplis     = mktemp("tmp$mkm-lis")
tmpcoord   = mktemp("tmp$mkm-coord")

# NOTE: The following has alread been done
#for(i=1;i<=13;i+=1) {
#  distcor[i] = 1000.
#}

if(n_yshift !=0 || n_xshift !=0) {
  imshift(n_image,n_shiftim,n_xshift,n_yshift,interp_type="linear",
          boundary_typ="constant",constant=20000)
  n_image=n_shiftim
}
if(n_xbin < 1 || n_xbin > 2 || n_xbin != n_ybin) {
  printf("%dx%d binning not an allowed.",n_xbin,n_ybin)
  printf("Must be 1x1 or 2x2\n")
  bye
} 
n_binning = n_xbin

## Check image scale size
# BUG: Modify so a simple bias subtraction is performed if not already done
# BUG: The following 1x1 mode is not expected and is not correct (Nov 2000)
imgets(n_image,"i_naxis1")
n_xim=real(imgets.value)
imgets(n_image,"i_naxis2")
n_yim=real(imgets.value)
if(n_binning == 2 && (n_yim != 512 || n_xim != 568)) {
  print("Image size not allowed.")
  print("Please use only trimmed amode (568x512) readouts.")
  bye
} else if (n_binning == 1 && (n_yim !=1024 || n_xim != 3104)) {
  print("Image size not allowed.")
  print("Please use only trimmed full format (3104x1024) readouts")
  bye
}

## change contrast where baffles block light
# those commented out have been superceded
# 2002 June 3 based on image from 2002 Apr 12
# BUG: The imcalc string has a very limited length which makes things hard
#      this can be fixed by putting the expr in a file and ref'ing with @.
#      see the imcalc manpage for details
if(n_binning == 2) {
  #Print imcalc string into a variable
  imcalstr = "if (y<"//str(sbt[1])//")||(y>"//str(stp[13])//")||"
  for (i=1;i<13;i+=1) {
    sltop = int(stp[i])
    slbot = int(sbt[i+1])
    imcalstr = imcalstr//"(y>"//str(sltop)
    imcalstr = imcalstr//"&&y<"//str(slbot)//")||"
  }
  imcalstr = imcalstr//"(x<"//str(int(slminp))//")||"
  imcalstr = imcalstr//"(x>"//str(int(slmaxp))//") "
  imcalstr = imcalstr//"then (0.9*im1) else im1"
} else if(n_binning == 1) {
  print ("1x1 binning not yet implemented with image display")
  bye
#***   imcalstr = "if (y<sbt[1]*2.0) || (y>stp[13]*2.0) || "
#***   for (i=1;i<13;i+=1) {
#***     imcalstr = imcalstr//"(y>"//stp[i]*2.0//" && y>"//stp[i+1]*2.0//") || "
#***   } 
#***   imcalstr = imcalstr//"(x < 224) || (x > 942) then (im1 / 6.) else im1"
}
#print ("imcalcing -- "//imcalstr)
imcalc(n_image,tmpdispim,imcalstr)

## display image
if( auto_z )  {
  display(tmpdispim,1,zra+,zsc+) | scan(dispz1,dispz2)
  n_dispz1=real(substr(dispz1,4,strlen(dispz1)))
  n_dispz2=real(substr(dispz2,4,strlen(dispz2)))
} else  {
  display(tmpdispim, 1, zrange-, zscale-, z1=n_dispz1, z2=n_dispz2)
}
#print(n_dispz1//"  "//n_dispz2)

## Main selection loop
n_key  = "?"  # First iteration of loop will print help message
n_ycur = 0
n_xcur = 0
while (n_key != "q") { #|| (n_key != "Q")) {
  #if(n_ybin==1) n_ycur = n_ycur / 2.
  n_slit=0
  if (n_key=="c") {
    # Calculate centroid
    if (access(tmplis)) delete(tmplis, verify-)
    if (n_binning ==1 ) n_ycur=2. * n_ycur
    print(str(n_xcur)//"  "//str(n_ycur), >> tmplis)
    imcentroid(n_image,"",tmplis,verb-) | fields("STDIN","2,4") | \
      scan(n_xcen,n_ycen)
    if(access(tmplis)) delete(tmplis,verify-)
  } else {
    n_xcen = n_xcur
    n_ycen = n_ycur
  }
  for (i=1;i<=13;i+=1) {
    if (n_ycen > (sbt[i]-2) && n_ycen < (stp[i]+2)) n_slit = i
  }
  if (n_key=="h" || n_key=="c" || n_key=="l" || n_key=="-") {
    # Write array variables
    if(n_xcen < slminp || n_xcen > slmaxp || n_slit == 0) {
      if (n_key == "c") printf ("Centroid ")
      else printf ("Cursor ")
      printf("position out of slit unit range. Try again.\n")
      eflag = yes
    } else {
      a_xcur[n_slit] = n_xcur
      a_ycur[n_slit] = n_ycur
      a_xcen[n_slit] = n_xcen
      a_ycen[n_slit] = n_ycen
      a_key[n_slit]  = n_key
      eflag = no
    }
    # h and c are already handled just do l and -
    if(n_key == "l" && !eflag) {
      # l (longslit) puts the slitlet at the center position which is approx
      # the longslit position, hence the choice of letter
      n_xcen = slcenp
      a_xcen[n_slit] = -1.
      a_ycen[n_slit] = -1.
      printf("Slit %d is at the center position: ",n_slit)
      printf("%6.2f pix, 0.000 mm\n",slcenp)
      a_slposmm[n_slit] = 0.0
      a_xdcen[n_slit] = sze[n_slit] / n_binning
    } else if (n_key == "-" && !eflag) {
      # - puts the slitlet at its home/baffled position indicated by '-' in
      # the config file
      n_xcen = slbafp
      a_xcen[n_slit] = -2.   # In case this is used then undefined
      a_ycen[n_slit] = -2.
      a_slposmm[n_slit] = ((n_binning*slbafp)-sze[n_slit])*mmppix
      printf("Slit %d is at the home/baffled position: ",n_slit)
      printf("~%0.0f. pix, ~%0.1f mm\n",slbafp,a_slposmm[n_slit])
    } else if (!eflag) {
      # Calculate slit position in mm
      if(lrsimage) {
        distcor[n_slit] =  xco3[n_slit] * (n_xcen**3)
        distcor[n_slit] = distcor[n_slit] + (xco2[n_slit] * (n_xcen**2))
        distcor[n_slit] = distcor[n_slit] + (xco1[n_slit] * (n_xcen))
        distcor[n_slit] = distcor[n_slit] + (xco0[n_slit])
        a_xdcen[n_slit] = n_xcen
      } else {
        distcor[n_slit] = 0          # Undistorted mm pos of slitlet
        n_xdisto = n_xcen - het_optx                 #delta x
        n_ydisto = n_ycen - het_optx                 #delta y
        n_rdisto=sqrt((n_xdisto)**2 + (n_ydisto)**2) #delta r
        n_mdist=imdco2 * n_rdisto**3 / n_rdisto      #delta r'/delta r
        n_xdisto += n_xdisto * n_mdist               #del x += del x'
        n_xdisto += het_optx         # Distorted x coordinate
        a_xdcen[n_slit] = n_xdisto   # Distorted Pixel pos. of slitlet
      }
      # This is where the real work is done !!!!!!
      a_slposmm[n_slit] = sze[n_slit]-(n_binning*(n_xcen+distcor[n_slit]))
      a_slposmm[n_slit] = mmppix * a_slposmm[n_slit]
      printf("Undistorted slit %d X-coordinate: %0.2f+(%0.2f) pix, %0.3f mm\n",
             n_slit,n_xcen,distcor[n_slit],a_slposmm[n_slit])
    }  
    if (!eflag) {
      ## Now n_xcen is where we want to draw slitlet and n_slit is slitlet #
      # Erase an existing mask area
      n_xfrac=(((n_xcen-1) / 3104.) + 0.5 - (0.5 * (n_xim / 3104.)))
      #n_xfrac=(((n_xcen-1) / 1162.) + 0.5 - (0.5 * (n_xim / 1162.)))
      n_yfrac=(((n_slit_bottom[n_slit] + n_slit_bottom[n_slit + 1] -1)/ (n_binning *1024.)) + 0.5 - (0.5 * (n_yim / 1024.)))
print ("n_xfrac "//str(n_xfrac))
print ("n_yfrac "//str(n_yfrac))
print ("display "//tmpdispim//"["//(int(n_xcen) - 3)//":"//(int(n_xcen) + 3)//","//str(n_slit_bottom[n_slit])//":"//str(n_slit_bottom[n_slit + 1])//"]")
      display(tmpdispim//"["//(int(n_xcen) - 3)//":"//(int(n_xcen) + 3)//","//str(n_slit_bottom[n_slit])//":"//str(n_slit_bottom[n_slit + 1])//"]",
              1,erase-,xcen=n_xfrac,ycen=n_yfrac,zra-,zsc-,
              z1=n_dispz1,z2=n_dispz2)
      # draw new slitlet position
      if(access(tmpcoord)) delete(tmpcoord, ver-)
      for(i=n_slit_bottom[n_slit];i<=n_slit_bottom[n_slit + 1];i+=1) {
        print(str(int(int(n_xcen) - 1))//"   "//str(i), >> tmpcoord)
        print(str(int(int(n_xcen) + 1))//"   "//str(i), >> tmpcoord)
      }
      tvmark(1,tmpcoord,mark="point",pointsize=1,color=204,label-)
    }
  } else if(n_key == "?") {			 # close if h/c/m/- 
    ## Output help message
    printf("\nPosition cursor in a slit field of view, ")
    printf("and choose from the following\n")
    printf(" h - Place slitlet at the cursor position\n")
    printf(" c - Place slitlet at centroid of cursor object\n")
    printf(" l - Place slitlet at the center (0.000 mm) position\n")
    printf(" - - Place slitlet at its home/baffled position\n")
    printf(" ? - Display this help text\n")
    printf(" q - Quit out of slit assignment\n")
  } else {
    printf("Try again, get more sleep, and buy better tequila.\n")
  }
  n_junk2=fscan(imcur,n_xcur,n_ycur,n_junk,n_key) 
}  # close of while loop  

## Output information to screen and send unassigned slitlets to baffled pos.
# BUG: image size must be less than 9999 pix and offset < +/-999.99 pix
printf("\nInput image: %s\n",n_imagename)
printf("Output data file: %s\n",n_mkmfile)
printf("Object name: %s\n",n_objname)
printf("Position Angle on sky: %0.3f degrees\n",pa)
printf("X shift: %0.1f pixels\n",n_xshift)
printf("Y shift: %0.1f pixels\n",n_yshift)
printf("Input binning: %1dx%1d\n",n_binning,n_binning)
printf("Output binning: 1x1\n")
printf("SlitNum  SlPosmm    CurX    CurY   CentX   CentY DistCor  DCentX ")
printf("key\n")
printf("              mm     pix     pix     pix     pix     pix     pix\n")
for(i=1;i<=13;i+=1) {
  if(a_xcen[i] == -2.) {
    # Slitlet not assigned it will be baffled
    printf("     %2d -- Slitlet at home/baffled position\n",i)
  } else if(a_xcen[i] == -1.) {
    # Slitlet is at the center position
    printf("     %2d %8.3f -- Slitlet at center position\n",i,a_slposmm[i])
  } else {    
    printf("     %2d %8.3f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %s\n",
           i,a_slposmm[i],a_xcur[i],a_ycur[i],a_xcen[i],a_ycen[i],
           distcor[i],a_xdcen[i],a_key[i])
  }
}


## Redisplay image w/ faint MOS mask (maybe?)


## Choose reference object
n_key  = "?"  # First iteration of loop will print help message
n_ycur = 0
n_xcur = 0
n_rnum = 0
for (i=1;i<=5;i+=1) {
  a_xref[i] = -1.0  # -1.0 indicates there is no reference object
  a_yref[i] = -1.0
}
while (n_key != "q") { #|| (n_key != "Q")) {
  #if(n_ybin==1) n_ycur = n_ycur / 2.
  if (n_key=="c") {
    # Calculate centroid
    if (access(tmplis)) delete(tmplis, verify-)
    if (n_binning ==1 ) n_ycur=2. * n_ycur
    print(str(n_xcur)//"  "//str(n_ycur), >> tmplis)
    imcentroid(n_image,"",tmplis,verb-) | fields("STDIN","2,4") | \
      scan(n_xcen,n_ycen)
    if(access(tmplis)) delete(tmplis,verify-)
  } else {
    n_xcen = n_xcur
    n_ycen = n_ycur
  }
  if (n_key=="h" || n_key=="c" || n_key=="s") {
    if(n_key == "s") {  # Do stuff for s
      # s (slitlet) puts the reference object at the slitlet position
      n_slit=0
      for (i=1;i<=13;i+=1) {
        if (n_ycen > (sbt[i]-2) && n_ycen < (stp[i]+2)) n_slit = i
      }
      if(n_xcen < slminp || n_xcen > slmaxp || n_slit == 0) {
        printf("Cursor position out of slit unit range. Try again.\n")
        eflag = yes
      } else {
        if (a_xcen[n_slit] < 0.) {
          printf("No object found on slitlet %d. Try again\n",
                 n_slit)
          eflag = yes
        } else {
          n_rnum += 1
          n_xcen = a_xcen[n_slit]
          n_ycen = a_ycen[n_slit]
          eflag = no
        }
      }
    } else { # Do stuff for h and c
      n_rnum += 1
      eflag = no
    }
    if (!eflag) {
      ## Now n_xcen is where we want to draw reference object circle
      # draw reference object circle
      a_xref[n_rnum] = n_xcen
      a_yref[n_rnum] = n_ycen
      printf("Reference object %d at (%0.2f,%0.2f)\n",n_rnum,n_xcen,n_ycen)
      if(access(tmpcoord)) delete(tmpcoord, ver-)
      print (str(int(n_xcen))//"   "//str(int(n_ycen)),>>tmpcoord)
      tvmark(1,tmpcoord,mark="circle",radii="3,6",color=204,label-)
    }
  } else if(n_key == "r") {                      # plot a radial profile
    pradprof (n_file,n_xcur,n_ycur)    
  } else if(n_key == "?") {			 # close if h/c/m/- 
    ## Output help message
    printf("\nPosition cursor on a reference object, ")
    printf("and choose from the following\n")
    printf("(You may have up to 5)\n")
    printf(" h - Set reference object at cursor position\n")
    printf(" c - Set ref. obj. at centroid of cursor object\n")
    printf(" s - Set ref. obj. at object on slitlet\n")
    printf(" r - Plot a radial profile at cursor position\n")
    printf(" ? - Display this help text\n")
    printf(" q - Quit out of slit assignment\n")
  } else {
    printf("Try again, get more sleep, and buy better scotch.\n")
  }
  if (n_rnum < 5) {
    n_junk2=fscan(imcur,n_xcur,n_ycur,n_junk,n_key) 
  } else {
    n_key="q"
  }
}  # close of while loop  

## Table output to copy of input file added as a .fits extension to it, but 
## if one exists, delete and replace it.
# Create temporary table locations
tmpsltab      = mktemp("tmp$mkm-sl")
tmpcoeftab    = mktemp("tmp$mkm-coef")
tmpreftab     = mktemp("tmp$mkm-ref")
tmptdata      = mktemp("tmp$mkm-tdata")

# Slit table generation
if(access(tmptdata)) delete(tmptdata,verify-)
printf("# Makemos %s slitlet position file\n",verstring,>>tmptdata)
printf("# Generated: ",>>tmptdata)
date(,>>tmptdata)
printf("# Based on: msetscr$mkm-slits-5.0.0.cd\n",>>tmptdata)
printf("# SlitNum  SlPosmm    CurX    CurY   CentX   CentY DistCor  DCentX ",
       >>tmptdata)
printf("key\n",>>tmptdata)
printf("#               mm     pix     pix     pix     pix     pix     pix ",
       >>tmptdata)
printf("   \n",>>tmptdata)
for(i=1;i<=13;i+=1) {
  # Output slit information for all working slits
  printf("       %2d %8.3f %7.2f %7.2f %7.2f %7.2f %7.2f %7.2f %s\n",
         i,a_slposmm[i],a_xcur[i],a_ycur[i],a_xcen[i],a_ycen[i],
         distcor[i],a_xdcen[i],a_key[i],>>tmptdata)
}
tcreate (tmpsltab,"msetscr$mkm-slits-5.0.0.cd",tmptdata)
# Add header parameters to slit position table
# TABNAME - Name of table
# MKMVERS - Makemos version string
# BINNING - binning of input data
# XSHIFT  - X shift (in pixels) applied to input image
# YSHIFT  - Y shift (in pixels) applied to input image
# Z1      - Image display z1
# Z2      - Image display z2
# CFGFILE - Configuration file name
# PROJID  - Project ID (e.g. UT02-2-014)
# OBJNAME - Input field object name
# INNAME  - Input image file name
# LRSIMG  - Was the input image an LRS image
# FLDRA   - Input field center RA
# FLDDEC  - Input field center DEC
# FLDEPO  - Epoch of image
# FLDPA   - Position angle on sky (increasing N thru E) of an increasing column
#           at the center of the field
thedit (tmpsltab,"TABNAME","MOS slitlet positions - "//verstring,show-)
thedit (tmpsltab,"MKMVERS","makemos.cl -- "//verstring,show-)
thedit (tmpsltab,"BINNING",n_binning,show-)
thedit (tmpsltab,"XSHIFT",n_xshift,show-)
thedit (tmpsltab,"YSHIFT",n_yshift,show-)
thedit (tmpsltab,"Z1",n_dispz1,show-)
thedit (tmpsltab,"Z2",n_dispz2,show-)
thedit (tmpsltab,"CFGFILE",n_cfgfile,show-)
thedit (tmpsltab,"PROJID",n_projid,show-)
thedit (tmpsltab,"OBJNAME",n_objname,show-)
thedit (tmpsltab,"INNAME",n_file,show-)
if (lrsimage) thedit (tmpsltab,"LRSIMG","LRS",show-)
else thedit (tmpsltab,"LRSIMG","OTHER",show-)
thedit (tmpsltab,"FLDRA",ra,show-)
thedit (tmpsltab,"FLDDEC",dec,show-)
thedit (tmpsltab,"FLDEPO",epoch,show-)
thedit (tmpsltab,"FLDPA",pa,show-)


## Reference object table generation
if(access(tmptdata)) delete(tmptdata,verify-)
printf("# Makemos %s reference object data file\n",verstring,>>tmptdata)
printf("# Generated: ",>>tmptdata)
date(,>>tmptdata)
printf("# Based on: msetscr$mkm-ref-5.0.0.cd\n",>>tmptdata)
printf("# ObjNum   CentX   CentY DistrtX DistrtY\n",>>tmptdata)
printf("#            pix     pix     pix     pix\n",>>tmptdata)
for (i=1;i<=n_rnum;i+=1) {
  n_xdisto = a_xref[i]
  n_ydisto = a_yref[i]
  printf ("      %2d %7.2f %7.2f ",i,n_xdisto,n_ydisto,>>tmptdata)
  if (!lrsimage) {
    n_xcen = n_xdisto - het_optx               #delta x
    n_ycen = n_ydisto - het_opty               #delta y
    n_rdisto=sqrt((n_xcen)**2 + (n_ycen)**2)   #delta r
    n_mdist=imdco2 * n_rdisto**3 / n_rdisto    #ratio of dist r to undist r
    n_xdisto += ((n_xcen)*n_mdist)
    n_ydisto += ((n_ycen)*n_mdist)
  }
  printf ("%7.2f %7.2f\n",n_xdisto,n_ydisto,>>tmptdata)
}
tcreate (tmpreftab,"msetscr$mkm-ref-5.0.0.cd",tmptdata)
# Add header parameters to slit position table
# TABNAME - Name of table
# MKMVERS - Makemos version string
# BINNING - binning of input data
# XSHIFT  - X shift (in pixels) applied to input image
# YSHIFT  - Y shift (in pixels) applied to input image
# LRSIMG  - Was the input image an LRS image
# OBJNAME - Input field object name
thedit (tmpreftab,"TABNAME","MOS slitlet positions",show-)
thedit (tmpreftab,"MKMVERS","makemos.cl -- "//verstring,show-)
thedit (tmpreftab,"BINNING",n_binning,show-)
thedit (tmpreftab,"XSHIFT",n_xshift,show-)
thedit (tmpreftab,"YSHIFT",n_yshift,show-)
if (lrsimage) thedit (tmpsltab,"LRSIMG","LRS",show-)
else thedit (tmpsltab,"LRSIMG","OTHER",show-)
thedit (tmpreftab,"OBJNAME",n_objname,show-)


# Distortion coefficient table generation
# BUG: output coefficients are a mix of binned and unbinned numbers and
#      don't necessarily conform to "Output binning: 1x1"
if(access(tmptdata)) delete(tmptdata,verify-)
printf("# Makemos %s distortion correction data file\n",verstring,>>tmptdata)
printf("# Generated: ",>>tmptdata)
date(,>>tmptdata)
printf("# Based on: msetscr$mkm-coef-5.0.0.cd\n",>>tmptdata)
printf("# SlitNum   Xco0      Xco1      Xco2       Xco3 ",>>tmptdata)
printf(" SlBotm   SlTop  SlZero\n",>>tmptdata)
printf("#           dpix      dpix      dpix       dpix ",>>tmptdata)
printf(" 2x2pix  2x2pix  1x1pix\n",>>tmptdata)
if (lrsimage) {
  for (i=1;i<=13;i+=1) {
    printf("       %2d %6.4f %9.6f %9.4e %10.4e %7.2f %7.2f %7.2f\n",
           i,xco0[i],xco1[i],xco2[i],xco3[i],sbt[i],stp[i],sze[i],>>tmptdata)
  }
}
tcreate (tmpcoeftab,"msetscr$mkm-coef-5.0.0.cd",tmptdata)
# Add header parameters to coefficient table
# TABNAME - Name of table
# MKMVERS - Makemos version string
# ASPPIX  - Unbinned arcsec per pixel, scales linearly with binning
# MMPPIX  - Unbinned mm per pixel, scales linearly with binning
# IMDCO   - Unbinned LRS distortion coefficient, scales as binning**2
# OCENTX  - Unbinned X pixel coordinate of optical center
# OCENTY  - Unbinned Y pixelcoordinate of optical center
# ASPMM   - Detector independent HET plate scale (arcsec/mm)
# SLCENP  - Pixel position of slitlet when at center (0.000 mm) position
# SLMAXP - Upper limit in pixel space of MOS slitlet (~ -17.0 mm)
# SLMINP  - Lower limit in pixel space of MOS slitlet (~ +17.0 mm)
# SLBAFP  - Baffled/Home position of slitlets (approximate and estimated)
thedit (tmpcoeftab,"TABNAME","MOS setup coefficients",show-)
thedit (tmpcoeftab,"MKMVERS","makemos.cl -- "//verstring,show-)
thedit (tmpcoeftab,"ASPPIX",asppix,show-)
thedit (tmpcoeftab,"MMPPIX",mmppix,show-)
thedit (tmpcoeftab,"IMDCO",imdco1,show-)
thedit (tmpcoeftab,"OCENTX",ocent[1],show-)
thedit (tmpcoeftab,"OCENTY",ocent[2],show-)
thedit (tmpcoeftab,"ASPMM",aspmm,show-)
thedit (tmpcoeftab,"SLCENP",slcenp,show-)
thedit (tmpcoeftab,"SLMAXP",slmaxp,show-)
thedit (tmpcoeftab,"SLMINP",slminp,show-)
thedit (tmpcoeftab,"SLBAFP",slbafp,show-)


## Copy input image and output tables to output file
# Output file name has already been checked for non-existence
# delete the existing temporary display image and reuse the name
# These portions can be seen with commands like:
# imhead mkm-filename.fits[0] long+
# tprint mkm-filename.fits[1] (or [2] or [3])
# imcopy mkm-filename.fits[0] irafname.imh
if(access(tmpdispim//".fits"))  imdelete(tmpdispim, verify-)
if(access(tmpdispim//".imh"))   imdelete(tmpdispim, verify-)
#imcopy (n_file,tmpdispim//".hhh",verbose-) # Didn't work properly b/c BZERO
# This will do nothing if image is real, if int it will convert to real
imarith (n_file,"*",1.,tmpdispim//".hhh")   # Convert to real instead
hedit (tmpdispim//".hhh","MKMVERS","makemos.cl -- "//verstring,add+,ver-,show-)
stwfits (tmpdispim//".hhh,"//tmpsltab//".tab,"//tmpreftab//".tab,"//tmpcoeftab//".tab",n_mkmfile,ext+,gftoxdim-)
#stwfits (tmpdispim//".hhh,"//tmpsltab//".tab,"//tmpcoeftab//".tab",n_mkmfile,ext+,gftoxdim-)
imdelete(tmpdispim//".hhh",verify-)

## Delete temporary files
if(access(tmplis)) delete(tmplis,ver-)
if(access(n_shiftim//".fits"))  imdelete(n_shiftim, verify-)
if(access(n_shiftim//".imh"))   imdelete(n_shiftim, verify-)
if(access(tmpdispim//".fits"))  imdelete(tmpdispim, verify-)
if(access(tmpdispim//".imh"))   imdelete(tmpdispim, verify-)
if(access(tmpcoord))            delete(tmpcoord,ver-)
if(access(tmpsltab//".tab"))	delete(tmpsltab//".tab",verify-)
if(access(tmpcoeftab//".tab"))	delete(tmpcoeftab//".tab",verify-)
if(access(tmpreftab//".tab"))	delete(tmpreftab//".tab",verify-)
if(access(tmptdata))		delete(tmptdata,verify-)

} #end of if (!sourcefile)

end
