procedure mmoscfg()
# msetscr$mmoscfg.cl
# Package to read the table information stored in the output mkm image
#  It then generates a suitable MOS configuration file based on this
#  information.

# Created: 2002-Jun-6 by J. Tufts
# Last Modified: 2002-Jun-14 (JRT)

# Revision History:
# v.0.0.2 2002-Jun-14 (JRT)
#  - Fixed bug in output comments
# v.0.0.1 2002-Jun-06 (JRT)
#  - First development version, based on Marcel Bergmann's original makemos.cl

char	mkmfile  {"",prompt="MKM File with slit/object tables"}
char    cfgfile  {"",prompt="MOS configuration file name"}
bool	clobber  {no,prompt="Clobber output image file name"}

begin


## Variable definitions
char  n_mkmfile,n_cfgfile
char  verstring
bool  n_clobber
int   i
int   n_slits, n_refs, n_cslit, n_cref
# Parameters
int   p_i
real  p_r
char  p_c
bool  p_b

## Set the version string
verstring = "v.0.0.2 2002-Jun-14"

# Startup message
printf("mmoscfg.cl -- %s\n",verstring)
printf("-----------------------------------------------------------------\n\n")


## Set variables from parameters
n_mkmfile = mkmfile
n_cfgfile = cfgfile
n_clobber = clobber


## Determine actual output file name
if (n_mkmfile=="")      n_mkmfile = makemos.outname
if (!access(n_mkmfile)) {
  printf("ERROR: %s does not exist.\n Try using a full name with extension\n",
         n_mkmfile)
  bye
}
if (n_cfgfile=="auto")  n_cfgfile = n_mkmfile//".cfg"
if (n_cfgfile=="") {
  keypar (n_mkmfile//"[1]","CFGFILE",sil+)
  n_cfgfile = keypar.value
}


## Handle clobber of output file
if (n_clobber && access(n_cfgfile)) {
  delete(n_cfgfile, verify+)
}
if (!n_clobber && access(n_cfgfile)) {
  print("Configuration file "//n_cfgfile//" exists.  Task stopped.")
  bye
}


## Output configuration file comments
# BUG: output coefficients are a mix of binned and unbinned numbers and
#      don't necessarily conform to "Output binning: 1x1"
printf("# File created by mmoscfg.cl -- %s\n",verstring,>>n_cfgfile)
printf("# makemos.cl invented by Marcel Bergmann ",>>n_cfgfile)
  printf("(marcel@astro.as.utexas.edu)\n",>>n_cfgfile)
printf("# Configuration generated: ",>>n_cfgfile)
  date(,>>n_cfgfile)
keypar (n_mkmfile//"[1]","MKMVERS",sil+)   #Output makemos version number
printf("# %s\n",keypar.value,>>n_cfgfile)
printf("# All X output corrected for LRS distortion\n",>>n_cfgfile)
printf("# Output generated from: %s\n",n_mkmfile,>>n_cfgfile)
keypar (n_mkmfile//"[1]","INNAME",sil+)
printf("# Input image: %s\n",keypar.value,>>n_cfgfile)
keypar (n_mkmfile//"[1]","PROJID",sil+)
printf("# Project ID: %s\n",keypar.value,>>n_cfgfile)
keypar (n_mkmfile//"[1]","OBJNAME",sil+)
printf("# Object name: %s\n",keypar.value,>>n_cfgfile)
keypar (n_mkmfile//"[1]","FLDPA",sil+)
printf("# Position Angle on sky: %0.2f degrees\n",real(keypar.value),
       >>n_cfgfile)
keypar (n_mkmfile//"[1]","XSHIFT",sil+)
printf("# X shift: %0.1f pixels\n",real(keypar.value),>>n_cfgfile)
keypar (n_mkmfile//"[1]","YSHIFT",sil+)
printf("# Y shift: %0.1f pixels\n",real(keypar.value),>>n_cfgfile)
keypar (n_mkmfile//"[1]","LRSIMG",sil+)
if (keypar.value=="LRS") p_b=yes
else p_b=no
if (p_b) printf("# Generated from an LRS image\n",>>n_cfgfile)
else printf("# _NOT_ Generated from an LRS image\n",>>n_cfgfile)
keypar (n_mkmfile//"[1]","BINNING",sil+)
p_i = int(keypar.value)
printf("# Input binning: %dx%d\n",p_i,p_i,>>n_cfgfile)
printf("# Output binning: 2x2\n",>>n_cfgfile)
keypar (n_mkmfile//"[3]","MMPPIX",sil+)
p_r = real(keypar.value)
p_r = 0.5/p_r
printf("Output config file written in mm (%0.4f binned pix/mm assumed)\n",p_r)
printf("# Conversion of %0.4f binned pix/mm assumed\n",p_r,>>n_cfgfile)


## Output reference object comments
printf("# Reference objects\n",>>n_cfgfile)
printf("# ObjNum       X       Y\n",>>n_cfgfile)
keypar (n_mkmfile//"[2]","NAXIS2",sil+)
n_refs = int(keypar.value)
for(i=1;i<=n_refs;i+=1) {
  printf("#     %2d ",i,>>n_cfgfile)
  tabpar (n_mkmfile//"[2]","DistrtX",i)
  printf("%7.2f ",real(tabpar.value),>>n_cfgfile)
  tabpar (n_mkmfile//"[2]","DistrtY",i)
  printf("%7.2f\n",real(tabpar.value),>>n_cfgfile)
}


## Output slitlet setup information
printf("#\n",>>n_cfgfile)
printf("# SlitNum  SlPosmm          SlitCol\n",>>n_cfgfile)
printf("#               mm              pix\n",>>n_cfgfile)
#keypar (n_mkmfile//"[1]","NAXIS2",sil+)
#n_slits = int(keypar.value)
for(i=1;i<=13;i+=1) {
  printf("       %2d ",i,>>n_cfgfile)
  tabpar (n_mkmfile//"[1]","key",i)
  p_c = tabpar.value
  if (p_c == "*" || p_c == "-") {
    printf("       -\n",>>n_cfgfile)
  } else {
    tabpar (n_mkmfile//"[1]","SlPosmm",i)
    printf("%8.3f ",real(tabpar.value),>>n_cfgfile)
    tabpar (n_mkmfile//"[1]","DCentX",i)
    printf("       # %7.2f pix\n",real(tabpar.value),>>n_cfgfile)
  }
}

end
