procedure modmos(image,outimage)
# msetscr$modmos.cl
#This task will take an astrometrically correct image and put it into
#the proper format for the makemos HET LRS/MOS setup script.  This
#includes rotating the image to the desired PA, and centering it at the
#requested RA,DEC, as well as putting into the proper pixel scale for the HET

# Revision History:
# v.0.5.1 2002-Jul-16 (JRT)
#  - Modified startup message to be more clear as far as extensions go
# v.0.5.0 2002-Jun-13 (JRT)
#  - Modified to work with parameters set in msetup
#  - This script assumes assumes and generates binned setup images
#  - added verstring parameter
# version 0.3.0 2002-Mar-19 by J. Tufts
#  - Brought v.0.1.x and v.0.2.x trees into consistency
# version 0.2   2002-Mar-14 by M. Bergmann
#  - fix bug caused by image PA's near 0 where one axis had a PA around 360 
#    and the other was near 0.
# version 0.1.2 2002-Feb-13 by J. Tufts
#  - Didn't really manage to fix anything
#  - messed with atan rotation stuff
# version 0.1.1 2002Feb11 by J. Tufts
#  - Fixed some bugs with DSS images
#version 0.1
#12/17/01
#Marcel Bergmann
#marcel@astro.as.utexas.edu

#assumptions:
#		1) astrometric distortions of input image are negligible
#		2) pixels are square

#defaults:
#		1) PA=0
#		2) ra,dec at center of input image

char  image    {"",prompt="name of image to adapt"}
char  outimage {"",prompt="name for output image"}
char  ra       {"",prompt="RA for center of field, hh:mm:ss.s format"}
char  dec      {"",prompt="Dec for center of field, dd:mm:ss format"}
char  cent_x   {"",prompt="x pixel value for center of field"}
char  cent_y   {"",prompt="y pixel value for center of fieldxxx"}
real  pa       {"0.",prompt="Position Angle of output col. (deg. N through E)"}
bool  clobber  {no,prompt="clobber old output image?"}

begin

# Parameters initialized in msetup
real n_asppix, R2D, D2R
real het_optx, het_opty
# Procedure input parameters
char n_image
char n_outimage
char n_ra, n_dec
char n_cent_x, n_cent_y
real n_pa
bool n_clobber
# Internal procedure parameters
char n_tmpimage,n_tmpimage2    #,n_tmpimage3
real n_centx, n_centy
real n_inpixscale1, n_inpixscale2
real n_inpa1, n_inpa2
real n_difpa,n_avpa
real n_rot
int  n_rep
real n_xcen, n_ycen
real n_xlen, n_ylen
real n_cd11,n_cd12,n_cd21,n_cd22
real n_xmag,n_ymag
real thresh_dpa, thresh_psc
char verstring

# Set the version string
verstring     = "v.0.5.1 2002-Jul-16"

# Startup message
printf("modmos.cl -- %s\n\n",verstring)
printf("Must have negligable distortion, and square pixels\n")
printf("Must use full filename (including extensions) ")
printf("for both input and output\n\n")
printf("-----------------------------------------------------------------\n\n")


# Set error thresholds
thresh_dpa    = 0.5    # difference in PA warning threshold (in degrees)
thresh_psc    = 0.005  # difference in plate scale warning threshold

#The following parameters are now set in msetup (as of modmos-0.5.0)
#pi
#het_optx
#het_opty
#real het_pixscale  = 0.465  # ''/pix (rounded incorrect)
#real het_pixscale  = 0.471  # ''/pix (old distorted value)
#real het_pixscale  = 0.4646 # ''/pix # Now this is an unbinned value (0.5.0)
n_asppix      = asppix * 2.0    #''/ubpix * 2 ubpix/bpix
het_optx      = ocent[1] / 2.0  #ubpix / 2 ubpix/bpix
het_opty      = ocent[2] / 2.0
R2D           = 180. / pi       #degrees per radian
D2R           = pi / 180.       #radians per degree

n_image       = image
n_outimage    = outimage
n_ra          = ra
n_dec         = dec
n_cent_x      = cent_x
n_cent_y      = cent_y
n_clobber     = clobber
n_pa          = pa


## Check to see that the input file exists and has the full extension included
if (!access(n_image)) {
  printf("ERROR: %s does not exist.\n Try using a full name with extension\n",
         n_image)
  bye
}


## Handle clobber
if (n_clobber && access(n_outimage)) {
  delete(n_outimage, verify+)
}
if (access(n_outimage)) {
  print("Output image "//n_outimage//" exists.  Task stopped.")
  bye
}


## Check for existence of CD parameters
keypar(n_image,"CD1_1", >>& "/dev/null")
if(!keypar.found) {
  printf("Header astrometry keyword CD1_1 not found in %s. \n",n_image)
  printf("Task stopped.\n")
  bye
}
keypar(n_image,"CD1_2", >>& "/dev/null")
if(!keypar.found) {
  printf("Header astrometry keyword CD1_2 not found in %s. \n",n_image)
  printf("Task stopped.\n")
  bye
}
keypar(n_image,"CD2_1", >>& "/dev/null")
if(!keypar.found) {
  printf("Header astrometry keyword CD2_1 not found in %s. \n",n_image)
  printf("Task stopped.\n")
  bye
}
keypar(n_image,"CD2_2", >>& "/dev/null")
if(!keypar.found) {
  printf("Header astrometry keyword CD2_2 not found in %s. \n",n_image)
  printf("Task stopped.\n")
  bye
}


## first, make sure input image is >= than 568x512, or blkrep so that it is.
n_tmpimage=mktemp("modm-tmpim")
keypar(n_image,"i_naxis1")
n_xlen=real(keypar.value)
keypar(n_image,"i_naxis2")
n_ylen=real(keypar.value)
if(n_xlen<568 || n_ylen < 512) {
  n_rep=max(int(568./n_xlen),int(512./n_ylen)) + 1
  blkrep(n_image,n_tmpimage,n_rep,n_rep)
} else {
  n_rep=1
  imcopy(n_image,n_tmpimage, >>& "/dev/null")
}


## choose which coords to use for rotation point(ra/dec, x/y, or imcenter)
if(n_ra=="" && n_dec=="" && n_cent_x=="" && n_cent_y=="") {
  print("No field center given.  Will use center of input image")
  keypar(n_tmpimage,"i_naxis1")
  n_centx=(real(keypar.value) / 2.)
  keypar(n_tmpimage,"i_naxis2")
  n_centy=(real(keypar.value) / 2.)
} else if(n_ra!="" && n_dec!="" && n_cent_x!="" && n_cent_y!="") {
  print("Using RA="//n_ra//"  DEC="//n_dec//" for field center") 
  rd2xy(n_tmpimage,real(n_ra),real(n_dec),hour+,>>"/dev/null")
  n_centx=rd2xy.x
  n_centy=rd2xy.y
  keypar(n_tmpimage,"i_naxis1")
  n_xlen=real(keypar.value)
  keypar(n_tmpimage,"i_naxis2")
  n_ylen=real(keypar.value)
  if(n_centx < 1 || n_centy < 1 || n_centx > n_xlen || n_centy > n_ylen) {
    print("RA, DEC are not in input image boundaries.  Task stopped.")
    bye
  }
} else if(n_ra!="" && n_dec!="" && n_cent_x=="" && n_cent_y=="") {
  rd2xy(n_tmpimage,real(n_ra),real(n_dec),hour+,>>"/dev/null")
  n_centx=rd2xy.x
  n_centy=rd2xy.y
  keypar(n_tmpimage,"i_naxis1")
  n_xlen=real(keypar.value)
  keypar(n_tmpimage,"i_naxis2")
  n_ylen=real(keypar.value)
  if(n_centx < 1 || n_centy < 1 || n_centx > n_xlen || n_centy > n_ylen) {
    print("RA, DEC are not in input image boundaries.  Task stopped.")
    bye
  }
} else if(n_ra=="" && n_dec=="" && n_cent_x!="" && n_cent_y!="") {
  n_centx=real(n_rep)*real(n_cent_x)
  n_centy=real(n_rep)*real(n_cent_y)
  keypar(n_tmpimage,"i_naxis1")
  n_xlen=real(keypar.value)
  keypar(n_tmpimage,"i_naxis2")
  n_ylen=real(keypar.value)
  if(n_centx < 1 || n_centy < 1 || n_centx > n_xlen || n_centy > n_ylen) {
    printf("X, Y for field center are not in input image boundaries. ")
    printf("Task stopped.\n")
    bye
 }
} else {
  printf("Coordinates for output image center do not make sense. \n")
  printf("Task stopped.\n")
  bye
}


## Compute the input PA and pixel scale
keypar(n_tmpimage,"cd1_1")
n_cd11=real(keypar.value)
keypar(n_tmpimage,"cd1_2")
n_cd12=real(keypar.value)
keypar(n_tmpimage,"cd2_1")
n_cd21=real(keypar.value)
keypar(n_tmpimage,"cd2_2")
n_cd22=real(keypar.value)

n_inpixscale1=3600.*sqrt(n_cd11**2 + n_cd12**2)
n_inpixscale2=3600.*sqrt(n_cd21**2 + n_cd22**2)

#n_inpa1=180.-180./3.1415926*atan(n_cd12,n_cd11)
#n_inpa2=180./3.1415926*atan(n_cd21,n_cd22)
n_inpa1=atan(n_cd12,-1.*n_cd11)  # Compute image PA from RA axis (in radians)
n_inpa2=atan(-1.*n_cd21,n_cd22)  # Compute image PA from DEC axis (in radians)
n_difpa=abs(n_inpa1 - n_inpa2)

# kludge to fix pa1=0+/-delta pa2=2pi+/-delta. In a perfect world 
# n_inpa1=n_inpa2, but not all coordinate systems are orthogonal. Clearly
# these two measures of image PA can't differ by more than pi/2 or the coord
# system is imaginary.
if (n_difpa > (pi / 2)) {
  if (n_inpa1 > n_inpa2) {
    n_inpa1 = n_inpa1 - (2 * pi)
  }
  else {
    n_inpa2 = n_inpa2 - (2 * pi)
  }
  n_difpa=abs(n_inpa1 - n_inpa2) # Recalculate difpa for warning test later
}
n_avpa=(n_inpa1+n_inpa2)/2.

if (n_avpa<0) {                  # Convert to 0-2pi range fm. -pi to pi range
  n_avpa = n_avpa + (2 * pi)
}
n_avpa  = R2D * n_avpa    # Convert back to degrees
n_difpa = R2D * n_difpa  
n_inpa1 = R2D * n_inpa1
n_inpa2 = R2D * n_inpa2
printf("inpa1: %8.3f\t\tinpa2: %8.3f\ndifpa: %8.3f\t\t avpa: %8.3f\n"\
       ,n_inpa1,n_inpa2,n_difpa,n_avpa)

if(n_difpa > thresh_dpa) {
 printf ("WARNING: PA derived from two axes differs by more than %0.2f \
         degrees.\nAverage may be incorrect\n",thresh_dpa)
}
if(abs(n_inpixscale1-n_inpixscale2)/n_inpixscale1 > thresh_psc) {
 printf("WARNING: input pixels are out of square by more than 0.4.\n")
 printf("Check results\n")
}
n_xmag=n_asppix / n_inpixscale1
n_ymag=n_asppix / n_inpixscale2
printf(" xmag: %8.3f\t\t ymag: %8.3f\n",n_xmag,n_ymag)

#n_difpa=abs(n_inpa1 - n_inpa2)
#n_avpa=(n_inpa1+n_inpa2)/2.
#if(n_difpa > 359.5) {
# n_inpa2 = n_inpa2 - 360.
# n_difpa=abs(n_inpa1 - n_inpa2)
# n_avpa=(n_inpa1+n_inpa2)/2.
#}
#if(n_difpa > 0.5 && n_difpa < 359.5) {
# print("WARNING: PA derived from two axes differs by more than 0.5 degrees. Using average")
# printf("inpa1: %7.3f\t\tinpa2: %7.3f\ndifpa: %7.3f\t\tavpa: %7.3f\n",n_inpa1,n_inpa2,n_difpa,n_avpa)
#}
#if(abs(n_inpixscale1-n_inpixscale2)/n_inpixscale1 > 0.005) {
# print("WARNING: input pixels are not square.  Check results")
#}
#n_xmag=het_pixscale / n_inpixscale1
#n_ymag=het_pixscale / n_inpixscale2

n_rot = n_avpa - n_pa
printf("Rotating input by %0.2f - %0.2f: %0.2f deg. ",n_avpa,n_pa,n_rot)
if (n_rot < 0) {
  n_rot = n_rot + 360.
  printf ("(or +%0.2f deg.)",n_rot)
}
printf ("\n")

n_tmpimage2=mktemp("modm-tmpim2")
#n_tmpimage3=mktemp("modm-tmpim3")

# geotran command modified to work with msetup (0.1.3)
###geotran(n_tmpimage,n_tmpimage2,"",xin=n_centx,yin=n_centy,xout=het_optx,\
###        yout=het_opty,xmag=n_xmag,ymag=n_ymag,xrot=n_rot,yrot=n_rot,\
#        boundary="constant",const=0.,xscale=1,yscale=1,>>"/dev/null")
###        boundary="constant",const=0.,xscale=1,yscale=1)
geotran(n_tmpimage,n_tmpimage2,"",xin=n_centx,yin=n_centy,xout=het_optx,\
        yout=het_opty,xmag=n_xmag,ymag=n_ymag,xrot=n_rot,yrot=n_rot,\
        boundary="constant",const=0.,xscale=1,yscale=1)

imcopy(n_tmpimage2//"[1:568,1:512]",n_outimage, >>& "/dev/null")
imdelete(n_tmpimage,ver-)
imdelete(n_tmpimage2,ver-)

hedit(n_outimage,"POSANGLE",n_pa,upd+,ver-,add+,>>& "/dev/null")
hedit(n_outimage,"CCDSUM","2 2",upd+,ver-,add+,>>& "/dev/null")
hedit(n_outimage,"MODMVERS",verstring,upd+,ver-,add+,>>& "/dev/null")
# Put back in the CD parameters that were taken out b/c they were zero
keypar (n_outimage,"CD1_2",sil+)
if (!keypar.found) hedit(n_outimage,"CD1_2",0.,upd+,ver-,add+,>>& "/dev/null")
keypar (n_outimage,"CD2_1",sil+)
if (!keypar.found) hedit(n_outimage,"CD2_1",0.,upd+,ver-,add+,>>& "/dev/null")

end

